<?php

namespace Drupal\ohdear_integration;

use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Logger\LoggerChannelInterface;
use OhDear\PhpSdk\OhDear;

/**
 * Service for easier handling of OhDear php sdk.
 */
class OhDearSdkService {

  /**
   * The ohDear integration config.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $ohDearIntegrationSettings;

  /**
   * The logger.channel.ohdear_integration service.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;


  /**
   * The OhDear Sdk object.
   *
   * @var \OhDear\PhpSdk\OhDear
   */
  protected $ohdear;

  /**
   * The ohdear site id.
   *
   * @var int
   */
  protected $siteId;

  /**
   * The ohdear site.
   *
   * @var \OhDear\PhpSdk\Resources\Site
   */
  protected $ohDearSite;

  /**
   * Constructs an OhDearSdkService object.
   *
   * @param \Drupal\Core\Config\ImmutableConfig $ohdear_integration_settings
   *   The config factory.
   * @param \Drupal\Core\Logger\LoggerChannelInterface $ohdear_integration_logger
   *   The logger.channel.ohdear_integration service.
   */
  public function __construct(ImmutableConfig $ohdear_integration_settings, LoggerChannelInterface $ohdear_integration_logger) {
    $this->ohDearIntegrationSettings = $ohdear_integration_settings;
    $this->logger = $ohdear_integration_logger;
  }

  /**
   * Creates OhDear sdk object.
   *
   * @return \OhDear\PhpSdk\OhDear
   *   The OhDear php sdk object.
   */
  public function getOhDear() {
    if (isset($this->ohdear)) {
      return $this->ohdear;
    }
    if ($apiKey = $this->ohDearIntegrationSettings->get('ohdear_api_key')) {
      $this->ohdear = new OhDear($apiKey);
      return $this->ohdear;
    }
    $this->logger->error('OhDear API key is missing!');
    throw new \Exception('OhDear API key is missing.');
  }

  /**
   * Reads site id from the configuration.
   *
   * @return int
   *   The site id.
   *
   * @throws \Exception
   */
  public function getSiteId() {
    if (isset($this->siteId)) {
      return $this->siteId;
    }
    $siteId = (int) $this->ohDearIntegrationSettings->get('ohdear_site_id');
    if (!$siteId) {
      $this->logger->error('OhDear site id is missing! Fix it at <a href="/admin/config/system/ohdear-settings">OhDear Integration configuration form</a>.');
      throw new \Exception('OhDear site id is missing.');
    }
    $this->siteId = (int) $siteId;
    return $this->siteId;
  }

  /**
   * Returns the OhDear php sdk site object.
   *
   * @return \OhDear\PhpSdk\Resources\Site
   *   The OhDear site for current site id.
   *
   * @throws \Exception
   */
  public function getSite() {
    if (isset($this->ohDearSite)) {
      return $this->ohDearSite;
    }
    $site = $this->ohdear->site($this->getSiteId());
    if (!$site) {
      $this->logger->error('OhDear site not found! Fix it at <a href="/admin/config/system/ohdear-settings">OhDear Integration configuration form</a>.');
      throw new \Exception('OhDear site not found.');
    }
    $this->ohDearSite = $site;
    return $this->ohDearSite;
  }

}
