<?php

namespace Drupal\Tests\ohdear_integration\Functional;

use Drupal\Component\Serialization\Json;
use Drupal\Tests\BrowserTestBase;
use PHPUnit\Framework\Assert;

/**
 * Test OhDear Integration endpoint.
 *
 * @group ohdear_integration
 */
class OhdearIntegrationEndpointTest extends BrowserTestBase {

  /**
   * A user with administrative permissions.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $adminUser;

  /**
   * Modules to install.
   *
   * @var array
   */
  protected static $modules = ['monitoring', 'monitoring_test', 'ohdear_integration', 'node'];

  /**
   * Secret to access ohdear integration endpoint.
   *
   * @var string
   */
  protected $ohdearSecret = '4zcke2vxfETABM8h';

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->config('ohdear_integration.settings')->set('ohdear_healthcheck_secret', $this->ohdearSecret)->save();

    // Create admin user.
    $this->adminUser = $this->drupalCreateUser([
      'access administration pages',
      'monitoring reports',
    ]);
  }

  /**
   * Testing healthcheck results json endpoint.
   */
  public function testApiResponseData() {
    // Anonymous request.
    $this->drupalGet('/json/oh-dear-health-check-results');
    Assert::assertEquals(403, $this->getSession()->getStatusCode(), 'Status code for anonymous request is not 403.');
    // Anonymous request with header secret.
    $this->drupalGet('/json/oh-dear-health-check-results', [], ['oh-dear-health-check-secret' => $this->ohdearSecret]);
    Assert::assertEquals(200, $this->getSession()->getStatusCode(), 'Status code for anonymous request with ohdear header authorization is not 200.');
    $data = Json::decode((string) $this->getSession()->getPage()->getContent());
    Assert::assertNotEmpty($data['checkResults'], 'Check results are missing.');
    $checkResults = $this->parseCheckResults($data);
    Assert::assertContains('test_sensor_exceeds', array_keys($checkResults), 'Test sensor exceeds not found.');
    // Anonymous request with wrong header secret.
    $this->drupalGet('/json/oh-dear-health-check-results', [], ['oh-dear-health-check-secret' => '111li3uygIRABaaa']);
    Assert::assertEquals(403, $this->getSession()->getStatusCode(), 'Status code for anonymous request with wrong header secret is not 403.');
    // Administrator account should also have access.
    $this->drupalLogin($this->adminUser);
    $this->drupalGet('/json/oh-dear-health-check-results');
    Assert::assertEquals(200, $this->getSession()->getStatusCode(), 'Status code for admin user request is not 200.');
    $data = Json::decode((string) $this->getSession()->getPage()->getContent());
    Assert::assertNotEmpty($data['checkResults'], 'Check results are missing.');
    $checkResults = $this->parseCheckResults($data);
    Assert::assertContains('test_sensor', array_keys($checkResults), 'Test sensor not found');
  }

  /**
   * Parse endpoint data response.
   *
   * Gives us option to add more detailed checks of sensors.
   *
   * @param array $data
   *   Json response of healthcheck endpoint.
   *
   * @return array
   *   Array of healthcheck results keyed with sensor name.
   */
  protected function parseCheckResults(array $data) {
    $checkResults = [];
    foreach ($data['checkResults'] as $checkResult) {
      $checkResults[$checkResult['name']] = $checkResult;
    }
    return $checkResults;
  }

}
